from typing import List

from beam.base import AbstractDataLoader, BaseConfigurationLoader
from beam.dataclass import FileConfiguration
from beam.types import OutputType


class File(BaseConfigurationLoader):
    def __init__(self, path: str, name: str, output_type: str):
        self.config: FileConfiguration = FileConfiguration(
            path=path, name=name, output_type=output_type
        )


class OutputManager(AbstractDataLoader):
    def __init__(self) -> None:
        self.files: List[File] = []
        self.dirs: List[File] = []

    def File(self, path: str, name: str, **_):
        self.files.append(File(path=path, name=name, output_type=OutputType.File))

    def Dir(self, path: str, name: str, **_):
        self.dirs.append(File(path=path, name=name, output_type=OutputType.Directory))

    def dumps(self):
        return [
            *[f.dumps() for f in self.files],
            *[d.dumps() for d in self.dirs],
        ]

    def from_config(self, outputs: List[dict]):
        if outputs is None:
            return

        for f in outputs:
            if f.get("output_type") == OutputType.Directory:
                self.Dir(**f)
            else:
                self.File(**f)



from marshmallow import Schema, fields, validate, ValidationError
from beam.exceptions import BeamSerializationError
from beam.types import PythonVersion


class BaseSerializer(Schema):
    def validate_and_dump(self, data):
        validation_errors = self.validate(data)

        if len(validation_errors) > 0:
            raise BeamSerializationError(
                f"{self.__class__.__name__}\n{validation_errors}"
            )
        return self.dump(dict)

    def validate(self, data, raise_exception=False, **kwargs):
        if raise_exception:
            validation_errors = self.validate(data)

            if len(validation_errors) > 0:
                raise BeamSerializationError(
                    f"{self.__class__.__name__}\n{validation_errors}"
                )
        return super().validate(data, **kwargs)

    class Meta:
        ordered = True


class AppSpecConfigurationSerializer(BaseSerializer):
    name = fields.String(required=True, validate=validate.Length(max=128))
    cpu = fields.String(required=True)
    gpu = fields.Integer(required=True)
    memory = fields.String(required=True)
    apt_install = fields.List(fields.String(), required=True)
    python_version = fields.String(
        validate=validate.OneOf(
            choices=[version[1] for version in PythonVersion.Types]
        ),
        required=True,
    )
    python_packages = fields.List(fields.String(), required=True)
    workspace = fields.String(required=True)


class BaseTriggerSerializer(BaseSerializer):
    inputs = fields.Dict(
        keys=fields.String(),
        values=fields.Dict(),
    )
    outputs = fields.Dict(
        keys=fields.String(),
        values=fields.Dict(),
    )
    handler = fields.Method(deserialize="validate_handler")
    trigger_type = fields.String()

    def raise_handler_validation_error(self):
        raise ValidationError("must be a string in form {pathToFile}:{method}")

    def validate_handler(self, handler):
        if not isinstance(handler, str):
            self.raise_handler_validation_error()

        parsed_handler = handler.split(":")

        if len(parsed_handler) != 2:
            self.raise_handler_validation_error()


class CronJobTriggerSerializer(BaseTriggerSerializer):
    # TODO: Validation step needs to be handled somewhere
    cron_schedule = fields.String()



from typing import Dict, List
from dataclasses import dataclass
from beam.types import OutputType, PythonVersion, TypeSerializer
from beam.base import (
    BaseConfigClass,
    BaseTriggerConfigClass,
)


@dataclass()
class AppSpecConfiguration(BaseConfigClass):
    name: str
    cpu: str
    gpu: int
    memory: str
    apt_install: PythonVersion
    python_version: List[str]
    python_packages: List[str]
    workspace: str


@dataclass
class WebhookConfiguration(BaseTriggerConfigClass):
    inputs: Dict[str, TypeSerializer]
    handler: str
    trigger_type: str = "webhook"


@dataclass
class CronJobConfiguration(BaseTriggerConfigClass):
    inputs: Dict[str, TypeSerializer]
    cron_schedule: str
    handler: str
    trigger_type: str = "cron_job"


@dataclass
class RestAPIConfiguration(BaseTriggerConfigClass):
    inputs: Dict[str, TypeSerializer]
    outputs: Dict[str, TypeSerializer]
    handler: str
    trigger_type: str = "rest_api"


@dataclass
class FileConfiguration(BaseConfigClass):
    path: str
    name: str
    output_type: OutputType



import json
import beam
import pytest

from beam.types import PythonVersion
from beam.utils import compose_cpu, compose_memory


def valid_reconstruction(app: beam.App):
    dumped_app = app.dumps()
    dumped_json = json.dumps(dumped_app)
    app_from_dumped_config = beam.App.from_config(json.loads(dumped_json))
    return app_from_dumped_config.dumps() == dumped_app


def test_basic_app_reconstruction():
    app = beam.App(
        name="some_app",
        cpu="4000m",
        memory="128mi",
        gpu=5,
        python_packages=["vim"],
        python_version=PythonVersion.Python36,
        workspace="./some_folder",
    )

    assert valid_reconstruction(app)


def test_adding_cron_trigger_correctly_should_succeed():
    app1 = beam.App(
        name="some_app",
        cpu=1,
        memory="128mi",
        gpu=5,
    )

    app1.Trigger.CronJob(
        inputs={
            "some_input": beam.Types.String(),
        },
        cron_schedule="asda",
        handler="test.py:app",
    )

    app2 = beam.App(
        name="some_app",
        cpu=1,
        memory="128mi",
        gpu=5,
    )

    app2.Trigger.Webhook(
        inputs={
            "some_input": beam.Types.String(),
        },
        handler="test.py:app",
    )

    app3 = beam.App(
        name="some_app",
        cpu=1,
        memory="128mi",
        gpu=5,
    )

    app3.Trigger.RestAPI(
        inputs={
            "some_input": beam.Types.String(),
        },
        outputs={
            "some_output": beam.Types.String(),
        },
        handler="test.py:app",
    )


def test_multiple_triggers_should_fail_build():
    app = beam.App(
        name="some_app",
        cpu=1,
        memory="128mi",
        gpu=5,
    )

    app.Trigger.CronJob(
        inputs={
            "some_input": beam.Types.String(max_length=128),
        },
        cron_schedule="asda",
        handler="test.py:app",
    )

    with pytest.raises(ValueError):
        app.Trigger.Webhook(
            inputs={
                "some_input": beam.Types.String(),
            },
            handler="test.py:app",
        )


def test_compose_cpu():
    assert compose_cpu(1) == "1000m"
    assert compose_cpu(0.25) == "250m"
    assert compose_cpu(1.999) == "1999m"
    assert compose_cpu("1000m") == "1000m"


def test_compose_memory():
    assert compose_memory("10gi") == "10Gi"
    assert compose_memory("10 gi ") == "10Gi"
    assert compose_memory("10 Gi ") == "10Gi"

    # Raises if Gi is > 256
    with pytest.raises(ValueError):
        compose_memory("10000Gi")

    assert compose_memory("256mi") == "256Mi"
    assert compose_memory("256 mi ") == "256Mi"
    assert compose_memory("256 Mi ") == "256Mi"

    # Raises if Mi is < 128 or >= 1000
    with pytest.raises(ValueError):
        compose_memory("127Mi")

    with pytest.raises(ValueError):
        compose_memory("1000Mi")

    # Test invalid formats
    with pytest.raises(ValueError):
        compose_memory("1000ti")

    with pytest.raises(ValueError):
        compose_memory("1000.0")

    with pytest.raises(ValueError):
        compose_memory("1000")

    with pytest.raises(ValueError):
        compose_memory("2 gigabytes")


def test_cpu_in_app_config():
    config = {"name": "some_app", "memory": "128mi"}

    app1 = beam.App(
        **config,
        cpu=4,
    )

    app2 = beam.App(
        **config,
        cpu="4000m",
    )

    assert valid_reconstruction(app1)
    assert valid_reconstruction(app2)
    assert app1.dumps() == app2.dumps()

    app3 = beam.App(
        **config,
        cpu=0.25,
    )

    app4 = beam.App(**config, cpu="250m")

    assert valid_reconstruction(app3)
    assert valid_reconstruction(app4)
    assert app3.dumps() == app4.dumps()



from abc import abstractmethod
from copy import deepcopy
from dataclasses import dataclass
from typing import Dict
from beam.types import TypeSerializer, Types


class BaseConfigClass:
    def to_dict(self):
        config = deepcopy(self)
        return config.__dict__

class BaseTriggerConfigClass(BaseConfigClass):
    inputs: Dict[str, TypeSerializer]
    outputs: Dict[str, TypeSerializer]
    handler: str
    trigger_type: str

    def to_dict(self):
        config = deepcopy(self)

        if hasattr(config, "inputs"):
            config.inputs = Types.dump_schema(config.inputs)

        if hasattr(config, "outputs"):
            config.outputs = Types.dump_schema(config.outputs)

        return config.__dict__


class BaseConfigurationLoader:
    config: BaseConfigClass

    def dumps(self):
        return self.config.to_dict()


class AbstractDataLoader:
    @abstractmethod
    def dumps(self):
        pass

    @abstractmethod
    def from_config(self):
        pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['beam', 'beam.configs', 'beam.scripts', 'beam.templates', 'beam.tests']

package_data = \
{'': ['*']}

install_requires = \
['marshmallow==3.18.0', 'mypy>=0.981,<0.982', 'packaging>=21.3,<22.0']

setup_kwargs = {
    'name': 'beam-sdk',
    'version': '0.7.5',
    'description': '',
    'long_description': 'None',
    'author': 'luke lombardi',
    'author_email': 'luke@slai.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
